//
// Created by zx on 2020/6/18.
//

#include "Locate_communicator.h"

Locate_communicator::Locate_communicator()
{
}
Locate_communicator::~Locate_communicator()
{
}

Error_manager Locate_communicator::encapsulate_msg(Communication_message* message)
{
    Error_manager code;
    //记录请求
    switch (message->get_message_type())
    {
        case Communication_message::eLocate_request_msg:
        {
            message::Measure_request_msg request;
            request.ParseFromString(message->get_message_buf());
            m_response_table[request]=message::Measure_response_msg();
            //发送请求
            code= Communication_socket_base::encapsulate_msg(message);
            if(code!=SUCCESS)
            {
                m_response_table.erase(request);
            }
            break;
        }
        default:
            code= Error_manager(FAILED,CRITICAL_ERROR," measure发送任务类型不存在");
            break;
    }
    return code;

}

Error_manager Locate_communicator::locate_request(message::Measure_request_msg& request,message::Measure_response_msg& result)
{
    /*
     * 检查request合法性,以及模块状态
     */

    if(request.base_info().sender()!=message::eMain||request.base_info().receiver()!=message::eMeasurer)
        return Error_manager(LOCATER_MSG_REQUEST_INVALID,MINOR_ERROR,"measure request invalid");

    if(m_response_table.find(request)==true)
        return Error_manager(LOCATER_MSG_REQUEST_REPEATED,MAJOR_ERROR," measure reques repeated");
    //设置超时,若没有设置,默认3000
    int timeout=request.base_info().has_timeout_ms()?request.base_info().timeout_ms():3000;
    //向测量节点发送测量请求,并记录请求

    Error_manager code;
    Communication_message message;

    message::Base_info base_msg;
    base_msg.set_msg_type(message::eLocate_request_msg);
    base_msg.set_sender(message::eMain);
    base_msg.set_receiver(message::eMeasurer);
    base_msg.set_timeout_ms(timeout);
    message.reset(base_msg,request.SerializeAsString());

    code=encapsulate_msg(&message);
    if(code!=SUCCESS)
        return code;
    //循环查询请求是否被处理
    auto start_time=std::chrono::system_clock::now();
    double time=0;
    do{
        //查询到记录
        message::Measure_response_msg response;
        ///查询是否存在,并且删除该记录,
        if(m_response_table.find(request,response))
        {
            //判断是否接收到回应,若回应信息被赋值则证明有回应
            if (response.has_base_info() && response.has_command_info())
            {
                message::Base_info response_base = response.base_info();
                //检查类型是否匹配
                if (response_base.msg_type() != message::eLocate_response_msg) {
                    return Error_manager(LOCATER_MSG_RESPONSE_TYPE_ERROR, MAJOR_ERROR,
                                         "measure response msg type error");
                }
                //检查基本信息是否匹配
                if (response_base.sender() != message::eMeasurer ||
                    response_base.receiver() != message::eMain ||
                    !(response.command_info() == request.command_info())) {
                    return Error_manager(LOCATER_MSG_RESPONSE_INFO_ERROR, MAJOR_ERROR,
                                         "measure response msg info error");
                }
                result = response;
                m_response_table.erase(request);

                return SUCCESS;

            }
        }
        else
        {
             //未查询到记录,任务已经被提前取消,记录被删除
             return Error_manager(LOCATER_MSG_REQUEST_CANCELED,MINOR_ERROR,"measure request canceled");
        }

        auto end_time=std::chrono::system_clock::now();
        auto duration = std::chrono::duration_cast<std::chrono::microseconds>(end_time - start_time);
        time=1000.0*double(duration.count()) * std::chrono::microseconds::period::num / std::chrono::microseconds::period::den;
        std::this_thread::yield();
        usleep(1000);
    }while(time<double(timeout));
    m_response_table.erase(request);
    return Error_manager(RESPONSE_TIMEOUT,MINOR_ERROR,"measure request timeout");
}

//检查消息是否可以被解析, 需要子类重载
Error_manager Locate_communicator::check_executer(Communication_message* p_msg)
{
    return SUCCESS;
}

Error_manager Locate_communicator::check_statu(int terminal_id)
{
    std::chrono::system_clock::time_point time_now=std::chrono::system_clock::now();
    auto durantion=time_now-m_statu_recv_time;
    if(m_measure_statu_msg.has_base_info()== false
        || durantion>std::chrono::seconds(5))
    {
        return Error_manager(ERROR,MINOR_ERROR,"测量节点通讯断开");
    }
    if(m_measure_statu_msg.laser_manager_status()==message::LASER_MANAGER_FAULT)
    {
        return Error_manager(ERROR,MINOR_ERROR,"测量模块扫描控制故障");
    }
    for(int i=0;i<m_measure_statu_msg.laser_statu_vector_size();++i)
    {
        message::Laser_statu laser_statu=m_measure_statu_msg.laser_statu_vector(i);
        if(laser_statu==message::LASER_FAULT || laser_statu==message::LASER_DISCONNECT)
        {
            return Error_manager(ERROR,MINOR_ERROR,"测量模块雷达故障或者断线");
        }
    }
    return SUCCESS;
}

//检查消息是否有效, 主要检查消息类型和接受者, 判断这条消息是不是给我的.
Error_manager Locate_communicator::check_msg(Communication_message*  p_msg)
{
    //通过 p_msg->get_message_type() 和 p_msg->get_receiver() 判断这条消息是不是给我的.
    //子类重载时, 增加自己模块的判断逻辑, 以后再写.
    if ( (p_msg->get_message_type() == Communication_message::Message_type::eLocate_response_msg
        ||p_msg->get_message_type() == Communication_message::Message_type::eLocate_status_msg)
         && p_msg->get_receiver() == Communication_message::Communicator::eMain )
    {
        return Error_code::SUCCESS;
    }
    else
    {
        //认为接受人
        return Error_code::INVALID_MESSAGE;
    }
}

Error_manager Locate_communicator::encapsulate_send_data(){
    return SUCCESS;
}

Error_manager Locate_communicator::execute_msg(Communication_message* p_msg)
{
    if(p_msg== nullptr)
        return Error_manager(POINTER_IS_NULL,CRITICAL_ERROR,"measure response msg pointer is null");


    //测量response消息
    switch (p_msg->get_message_type())
    {
        ///测量结果反馈消息
        case Communication_message::eLocate_response_msg:
        {
            message::Measure_response_msg response;
            response.ParseFromString(p_msg->get_message_buf());
            message::Measure_request_msg request=create_request_by_response(response);
            ///查询请求表是否存在,并且更新
            if(m_response_table.find_update(request,response)==false)
            {
                return Error_manager(LOCATER_MSG_RESPONSE_HAS_NO_REQUEST,NEGLIGIBLE_ERROR,"measure response without request");
            }

            break;
        }
        ///测量系统状态
        case Communication_message::eLocate_status_msg:
        {
            if(m_measure_statu_msg.ParseFromString(p_msg->get_message_buf())==false)
                return Error_manager(ERROR,CRITICAL_ERROR,"measure statu msg parse failed, CRITICAL_ERROR");
            m_statu_recv_time=std::chrono::system_clock::now();
            break;
        }
    }
    return SUCCESS;

}

Error_manager Locate_communicator::cancel_request(message::Measure_request_msg& request)
{
    if(m_response_table.find(request)==true)
        m_response_table.erase(request);
    return SUCCESS;
}

message::Measure_request_msg Locate_communicator::create_request_by_response(message::Measure_response_msg& response)
{
    message::Measure_request_msg request;
    message::Base_info baseInfo;
    baseInfo.set_msg_type(message::eLocate_request_msg);
    baseInfo.set_sender(response.base_info().receiver());
    baseInfo.set_receiver(response.base_info().sender());
    request.mutable_base_info()->CopyFrom(baseInfo);
    request.mutable_command_info()->CopyFrom(response.command_info());
    request.set_terminal_id(response.terminal_id());
    return request;
}

